import 'dart:convert';

import 'package:flutterurlpreview/model.dart';
import 'package:html/dom.dart';
import 'package:html/parser.dart';
import 'package:http/http.dart';

Future<LinkData> checkAndGetLinkData(String str) async {
  RegExp exp =
      new RegExp(r'(?:(?:https?|ftp):\/\/)?[\w/\-?=%.]+\.[\w/\-?=%.]+');
  final b = exp.hasMatch(str);
  if (b) {
    final response = await get(str);
    if (response.statusCode == 200) {
      final result = getOpenGraphDataFromResponse(response);
      return LinkData.fromJson(result);
    }
  }
  return null;
}

Map<String, dynamic> getOpenGraphDataFromResponse(Response response) {
  var requiredAttributes = ['title', 'image'];
  Map<String, dynamic> data = new Map<String, dynamic>();
  var document = parse(utf8.decode(response.bodyBytes));
  var openGraphMetaTags = document.head.querySelectorAll("[property*='og:']");
  openGraphMetaTags.forEach((element) {
    var ogTagTitle = element.attributes['property'].split("og:")[1];
    var ogTagValue = element.attributes['content'];
    if ((ogTagValue != null && ogTagValue != "") ||
        requiredAttributes.contains(ogTagTitle)) {
      if (ogTagValue == null || ogTagValue.length == 0) {
        ogTagValue = _scrapeAlternateToEmptyValue(ogTagTitle, document);
      }
      data[ogTagTitle] = ogTagValue;
    }
  });
  return data;
}

String _scrapeAlternateToEmptyValue(String tagTitle, Document document) {
  if (tagTitle == "title") {
    return document.head.getElementsByTagName("title")[0].text;
  }
  if (tagTitle == "image") {
    var images = document.body.getElementsByTagName("img");
    if (images.length > 0) {
      return images[0].attributes["src"];
    }
    return "";
  }
  return "";
}
